@@ -98,13 +98,13 @@ static void parse_relocs(Elf *elf, Elf_Data *relocs, Elf_Data *symbols,
 		symbol_name = elf_strptr(elf, symbol_sh_link, symbol.st_name);
 
 		reloc->offset = rel.r_offset;
-		reloc->name = strdup(symbol_name);
+		strncpy(reloc->name, symbol_name, sizeof(reloc->name)-1);
+		reloc->name[sizeof(reloc->name)-1] = 0;
 	}
 }
 
 void radeon_elf_read(const char *elf_data, unsigned elf_size,
-					struct radeon_shader_binary *binary,
-					unsigned debug)
+		     struct radeon_shader_binary *binary)
 {
 	char *elf_buffer;
 	Elf *elf;
@@ -124,7 +124,6 @@ void radeon_elf_read(const char *elf_data, unsigned elf_size,
 	elf = elf_memory(elf_buffer, elf_size);
 
 	elf_getshdrstrndx(elf, &section_str_index);
-	binary->disassembled = 0;
 
 	while ((section = elf_nextscn(elf, section))) {
 		const char *name;
@@ -145,12 +144,11 @@ void radeon_elf_read(const char *elf_data, unsigned elf_size,
 			binary->config_size = section_data->d_size;
 			binary->config = MALLOC(binary->config_size * sizeof(unsigned char));
 			memcpy(binary->config, section_data->d_buf, binary->config_size);
-		} else if (debug && !strcmp(name, ".AMDGPU.disasm")) {
-			binary->disassembled = 1;
+		} else if (!strcmp(name, ".AMDGPU.disasm")) {
+			/* Always read disassembly if it's available. */
 			section_data = elf_getdata(section, section_data);
-			fprintf(stderr, "\nShader Disassembly:\n\n");
-			fprintf(stderr, "%.*s\n", (int)section_data->d_size,
-						  (char *)section_data->d_buf);
+			binary->disasm_string = strndup(section_data->d_buf,
+							section_data->d_size);
 		} else if (!strncmp(name, ".rodata", 7)) {
 			section_data = elf_getdata(section, section_data);
 			binary->rodata_size = section_data->d_size;
@@ -197,26 +195,3 @@ const unsigned char *radeon_shader_binary_config_start(
 	}
 	return binary->config;
 }
-
-void radeon_shader_binary_free_relocs(struct radeon_shader_reloc *relocs,
-					unsigned reloc_count)
-{
-	unsigned i;
-	for (i = 0; i < reloc_count; i++) {
-		FREE(relocs[i].name);
-	}
-	FREE(relocs);
-}
-
-void radeon_shader_binary_free_members(struct radeon_shader_binary *binary,
-					unsigned free_relocs)
-{
-	FREE(binary->code);
-	FREE(binary->config);
-	FREE(binary->rodata);
-
-	if (free_relocs) {
-		radeon_shader_binary_free_relocs(binary->relocs,
-						binary->reloc_count);
-	}
-}
