@@ -31,13 +31,15 @@
 #include "blend_jit.h"
 #include "builder.h"
 #include "state_llvm.h"
-#include "llvm/IR/DataLayout.h"
 
 #include <sstream>
 
 // components with bit-widths <= the QUANTIZE_THRESHOLD will be quantized
 #define QUANTIZE_THRESHOLD 2
 
+using namespace llvm;
+using namespace SwrJit;
+
 //////////////////////////////////////////////////////////////////////////
 /// Interface to Jitting a blend shader
 //////////////////////////////////////////////////////////////////////////
@@ -725,12 +727,7 @@ struct BlendJit : public Builder
 
         JitManager::DumpToFile(blendFunc, "");
 
-#if HAVE_LLVM == 0x306
-        FunctionPassManager
-#else
-        llvm::legacy::FunctionPassManager
-#endif
-            passes(JM()->mpCurrentModule);
+        ::FunctionPassManager passes(JM()->mpCurrentModule);
 
         passes.add(createBreakCriticalEdgesPass());
         passes.add(createCFGSimplificationPass());
