@@ -23,10 +23,12 @@
 #include "nine_helpers.h"
 #include "nine_shader.h"
 
+#include "vertexdeclaration9.h"
 #include "vertexshader9.h"
 
 #include "device9.h"
 #include "pipe/p_context.h"
+#include "cso_cache/cso_context.h"
 
 #define DBG_CHANNEL DBG_VERTEXSHADER
 
@@ -61,11 +63,24 @@ NineVertexShader9_ctor( struct NineVertexShader9 *This,
     info.sampler_mask_shadow = 0x0;
     info.sampler_ps1xtypes = 0x0;
     info.fog_enable = 0;
+    info.point_size_min = 0;
+    info.point_size_max = 0;
+    info.swvp_on = !!(device->params.BehaviorFlags & D3DCREATE_SOFTWARE_VERTEXPROCESSING);
+    info.process_vertices = false;
 
     hr = nine_translate_shader(device, &info);
+    if (hr == D3DERR_INVALIDCALL &&
+        (device->params.BehaviorFlags & D3DCREATE_MIXED_VERTEXPROCESSING)) {
+        /* Retry with a swvp shader. It will require swvp to be on. */
+        info.swvp_on = true;
+        hr = nine_translate_shader(device, &info);
+    }
+    if (hr == D3DERR_INVALIDCALL)
+        ERR("Encountered buggy shader\n");
     if (FAILED(hr))
         return hr;
     This->byte_code.version = info.version;
+    This->swvp_only = info.swvp_on;
 
     This->byte_code.tokens = mem_dup(pFunction, info.byte_size);
     if (!This->byte_code.tokens)
@@ -74,7 +89,7 @@ NineVertexShader9_ctor( struct NineVertexShader9 *This,
 
     This->variant.cso = info.cso;
     This->last_cso = info.cso;
-    This->last_key = 0;
+    This->last_key = (uint32_t) (info.swvp_on << 9);
 
     This->const_used_size = info.const_used_size;
     This->lconstf = info.lconstf;
@@ -82,7 +97,7 @@ NineVertexShader9_ctor( struct NineVertexShader9 *This,
     This->position_t = info.position_t;
     This->point_size = info.point_size;
 
-    for (i = 0; i < info.num_inputs && i < Elements(This->input_map); ++i)
+    for (i = 0; i < info.num_inputs && i < ARRAY_SIZE(This->input_map); ++i)
         This->input_map[i].ndecl = info.input_map[i];
     This->num_inputs = i;
 
@@ -97,6 +112,7 @@ NineVertexShader9_dtor( struct NineVertexShader9 *This )
     if (This->base.device) {
         struct pipe_context *pipe = This->base.device->pipe;
         struct nine_shader_variant *var = &This->variant;
+        struct nine_shader_variant_so *var_so = &This->variant_so;
 
         do {
             if (var->cso) {
@@ -107,6 +123,13 @@ NineVertexShader9_dtor( struct NineVertexShader9 *This )
             var = var->next;
         } while (var);
 
+        while (var_so && var_so->vdecl) {
+            if (var_so->cso) {
+                cso_delete_vertex_shader(This->base.device->cso_sw, var_so->cso );
+            }
+            var_so = var_so->next;
+        }
+
         if (This->ff_cso) {
             if (This->ff_cso == This->base.device->state.cso.vs)
                 pipe->bind_vs_state(pipe, NULL);
@@ -114,6 +137,7 @@ NineVertexShader9_dtor( struct NineVertexShader9 *This )
         }
     }
     nine_shader_variants_free(&This->variant);
+    nine_shader_variants_so_free(&This->variant_so);
 
     FREE((void *)This->byte_code.tokens); /* const_cast */
 
@@ -145,7 +169,7 @@ void *
 NineVertexShader9_GetVariant( struct NineVertexShader9 *This )
 {
     void *cso;
-    uint32_t key;
+    uint64_t key;
 
     key = This->next_key;
     if (key == This->last_key)
@@ -163,6 +187,10 @@ NineVertexShader9_GetVariant( struct NineVertexShader9 *This )
         info.byte_code = This->byte_code.tokens;
         info.sampler_mask_shadow = key & 0xf;
         info.fog_enable = device->state.rs[D3DRS_FOGENABLE];
+        info.point_size_min = asfloat(device->state.rs[D3DRS_POINTSIZE_MIN]);
+        info.point_size_max = asfloat(device->state.rs[D3DRS_POINTSIZE_MAX]);
+        info.swvp_on = device->swvp;
+        info.process_vertices = false;
 
         hr = nine_translate_shader(This->base.device, &info);
         if (FAILED(hr))
@@ -177,6 +205,38 @@ NineVertexShader9_GetVariant( struct NineVertexShader9 *This )
     return cso;
 }
 
+void *
+NineVertexShader9_GetVariantProcessVertices( struct NineVertexShader9 *This,
+                                             struct NineVertexDeclaration9 *vdecl_out,
+                                             struct pipe_stream_output_info *so )
+{
+    struct nine_shader_info info;
+    HRESULT hr;
+    void *cso;
+
+    cso = nine_shader_variant_so_get(&This->variant_so, vdecl_out, so);
+    if (cso)
+        return cso;
+
+    info.type = PIPE_SHADER_VERTEX;
+    info.const_i_base = 0;
+    info.const_b_base = 0;
+    info.byte_code = This->byte_code.tokens;
+    info.sampler_mask_shadow = 0;
+    info.fog_enable = false;
+    info.point_size_min = 0;
+    info.point_size_max = 0;
+    info.swvp_on = true;
+    info.vdecl_out = vdecl_out;
+    info.process_vertices = true;
+    hr = nine_translate_shader(This->base.device, &info);
+    if (FAILED(hr))
+        return NULL;
+    *so = info.so;
+    nine_shader_variant_so_add(&This->variant_so, vdecl_out, so, info.cso);
+    return info.cso;
+}
+
 IDirect3DVertexShader9Vtbl NineVertexShader9_vtable = {
     (void *)NineUnknown_QueryInterface,
     (void *)NineUnknown_AddRef,
