@@ -114,6 +114,9 @@ blorp_alloc_binding_table(struct blorp_batch *batch, unsigned num_entries,
       surface_offsets[i] = surface_state.offset;
       surface_maps[i] = surface_state.map;
    }
+
+   if (!cmd_buffer->device->info.has_llc)
+      anv_state_clflush(bt_state);
 }
 
 static void *
@@ -132,6 +135,14 @@ blorp_alloc_vertex_buffer(struct blorp_batch *batch, uint32_t size,
    return vb_state.map;
 }
 
+static void
+blorp_flush_range(struct blorp_batch *batch, void *start, size_t size)
+{
+   struct anv_device *device = batch->blorp->driver_ctx;
+   if (!device->info.has_llc)
+      anv_clflush_range(start, size);
+}
+
 static void
 blorp_emit_urb_config(struct blorp_batch *batch, unsigned vs_entry_size)
 {
