@@ -152,7 +152,8 @@ extern bool
 _mesa_is_es3_color_renderable(GLenum internal_format);
 
 extern bool
-_mesa_is_es3_texture_filterable(GLenum internal_format);
+_mesa_is_es3_texture_filterable(const struct gl_context *ctx,
+                                GLenum internal_format);
 
 #ifdef __cplusplus
 }
